@@ -14,12 +14,22 @@
 
 #include <RcppEigen.h>
 
+#include <type_traits>
 #include <memory>
+#include <complex>
 
 template <class T> 
 class dynEigenMat {
+    
+    static_assert(std::is_same<T, double>::value || 
+                  std::is_same<T, float>::value ||
+                  std::is_same<T, int>::value ||
+                  std::is_same<T, std::complex<double> >::value ||
+                  std::is_same<T, std::complex<float> >::value,
+                  "some meaningful error message");
+    
     private:
-        int nr, orig_nr, nc, orig_nc, r_start, r_end, c_start, c_end;
+        int nr, orig_nr, nc, orig_nc, r_start, r_end, c_start, c_end, ctx_id;
         Rcpp::StringVector _colNames, _rowNames;
         // T* ptr;
         // Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>* raw_ptr;
@@ -28,13 +38,13 @@ class dynEigenMat {
         
     public:
         Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> A;
-        viennacl::matrix<T> *vclA;
+        // viennacl::matrix<T> *vclA;
         // Eigen::Block<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block;
         
         
         // initializers
         dynEigenMat() { }; // private default constructor
-        dynEigenMat(SEXP A_){
+        dynEigenMat(SEXP A_, const int ctx): ctx_id(ctx) {
             A = Rcpp::as<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A_);
             orig_nr = A.rows();
             orig_nc = A.cols();
@@ -47,7 +57,7 @@ class dynEigenMat {
             
             ptr = std::make_shared<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A);
         };
-        dynEigenMat(Eigen::Matrix<T, Eigen::Dynamic,Eigen::Dynamic> A_){
+        dynEigenMat(Eigen::Matrix<T, Eigen::Dynamic,Eigen::Dynamic> A_, const int ctx): ctx_id(ctx) {
             A = A_;
             orig_nr = A.rows();
             orig_nc = A.cols();
@@ -60,7 +70,7 @@ class dynEigenMat {
             
             ptr = std::make_shared<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A);
         };
-        dynEigenMat(int nr_in, int nc_in){
+        dynEigenMat(int nr_in, int nc_in, const int ctx): ctx_id(ctx) {
             A = Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>::Zero(nr_in, nc_in);
             orig_nr = nr_in;
             orig_nc = nc_in;
@@ -72,7 +82,7 @@ class dynEigenMat {
             c_end = nc_in;
             ptr = std::make_shared<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A);
         };
-        dynEigenMat(T scalar, int nr_in, int nc_in){
+        dynEigenMat(T scalar, int nr_in, int nc_in, const int ctx): ctx_id(ctx) {
             A = Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>::Constant(nr_in, nc_in, scalar);
             orig_nr = nr_in;
             orig_nc = nc_in;
@@ -84,11 +94,11 @@ class dynEigenMat {
             c_end = nc_in;
             ptr = std::make_shared<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A);
         };
-        dynEigenMat(
-            Eigen::Matrix<T, Eigen::Dynamic,Eigen::Dynamic> &A_, 
-            const int row_start, const int row_end,
-            const int col_start, const int col_end
-            );
+        // dynEigenMat(
+        //     Eigen::Matrix<T, Eigen::Dynamic,Eigen::Dynamic> &A_, 
+        //     const int row_start, const int row_end,
+        //     const int col_start, const int col_end,
+        //     const int ctx): ctx_id(ctx - 1) ;
         dynEigenMat(Rcpp::XPtr<dynEigenMat<T> > dynMat);
         
         // pointer access
@@ -98,9 +108,12 @@ class dynEigenMat {
         std::shared_ptr<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>> getHostPtr(){
             return ptr;
         };
-        viennacl::matrix<T>* getDevicePtr(){
-            return shptr.get(); 
+        std::shared_ptr<viennacl::matrix<T> > getDevicePtr(){
+            return shptr; 
         };
+        // viennacl::matrix<T>* getDevicePtr(){
+        //     return shptr.get(); 
+        // };
         
         // object values
         int nrow(){ return nr; }
@@ -144,6 +157,36 @@ class dynEigenMat {
             ptr = std::make_shared<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >(A);
         };
         
+        void setRow(SEXP value, const int idx){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            block.row(idx-1) = Rcpp::as<Eigen::Matrix<T, Eigen::Dynamic, 1> >(value);
+        }
+        void setCol(SEXP value, const int idx){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            block.col(idx-1) = Rcpp::as<Eigen::Matrix<T, Eigen::Dynamic, 1> >(value);
+        }
+        void setElement(SEXP value, const int nr, const int nc){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            block(nr-1, nc-1) = Rcpp::as<T>(value);
+        }
+        
+        Eigen::Matrix<T, Eigen::Dynamic, 1> getRow(const int idx){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            Eigen::Matrix<T, Eigen::Dynamic, 1> out = block.row(idx-1);
+            return out;
+        }
+        Eigen::Matrix<T, Eigen::Dynamic, 1> getCol(const int idx){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            Eigen::Matrix<T, Eigen::Dynamic, 1> out = block.col(idx-1);
+            return out;
+        }
+        T getElement(const int nr, const int nc){
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > block = this->data();
+            T out = block(nr-1, nc-1);
+            return out;
+        }
+        
+        
         // setting pointer explicitly
         // note - this will assume complete control of pointer and delete it as well
         // following deletion of wherever this is set
@@ -154,6 +197,15 @@ class dynEigenMat {
             ptr = ptr_;
         };
         
+        // set context index
+        void setContext(const int ctx){
+            ctx_id = ctx;
+        }
+        // get context index
+        int getContext(){
+            return ctx_id;
+        }
+        
         // get host data
         Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, 0, Eigen::OuterStride<> > data(){
             // Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > temp(ptr.get()->data(), orig_nr, orig_nc);
@@ -175,7 +227,7 @@ class dynEigenMat {
         };
         
         // get device data
-        viennacl::matrix<T> device_data(long ctx_id){
+        viennacl::matrix<T> device_data(){
             Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > temp(ptr.get()->data(), orig_nr, orig_nc);
             Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > ref = temp.block(r_start-1, c_start-1, r_end-r_start + 1, c_end-c_start + 1);
             Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, 0, Eigen::OuterStride<> > block(
@@ -193,12 +245,34 @@ class dynEigenMat {
             
             return vclMat;
         };
+        
+        // get device data
+        viennacl::matrix<T> device_data(long ctx_in){
+            Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > temp(ptr.get()->data(), orig_nr, orig_nc);
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > ref = temp.block(r_start-1, c_start-1, r_end-r_start + 1, c_end-c_start + 1);
+            Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, 0, Eigen::OuterStride<> > block(
+                    ref.data(), ref.rows(), ref.cols(),
+                    Eigen::OuterStride<>(ref.outerStride())
+            );
+            
+            const int M = block.cols();
+            const int K = block.rows();
+            
+            ctx_id = ctx_in;
+            
+            viennacl::context ctx(viennacl::ocl::get_context(ctx_id));
+            
+            viennacl::matrix<T> vclMat(K,M, ctx = ctx);
+            viennacl::copy(block, vclMat);
+            
+            return vclMat;
+        };
         viennacl::matrix<T> getDeviceData(){
-            return *vclA;
+            return *shptr.get();
         };
         
         // copy to device (w/in class)
-        void to_device(long ctx_id){
+        void to_device(){
             Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > temp(ptr.get()->data(), orig_nr, orig_nc);
             Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > ref = temp.block(r_start-1, c_start-1, r_end-r_start + 1, c_end-c_start + 1);
             Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, 0, Eigen::OuterStride<> > block(
@@ -211,8 +285,26 @@ class dynEigenMat {
             
             viennacl::context ctx(viennacl::ocl::get_context(ctx_id));
             
-            vclA = new viennacl::matrix<T>(K, M, ctx=ctx);
-            shptr.reset(vclA);
+            shptr.reset(new viennacl::matrix<T>(K, M, ctx=ctx));
+            
+            viennacl::copy(block, *shptr.get());
+        };
+        // copy to device (w/in class) - custom context
+        void to_device(long ctx_in){
+            Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > temp(ptr.get()->data(), orig_nr, orig_nc);
+            Eigen::Ref<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > ref = temp.block(r_start-1, c_start-1, r_end-r_start + 1, c_end-c_start + 1);
+            Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, 0, Eigen::OuterStride<> > block(
+                    ref.data(), ref.rows(), ref.cols(),
+                    Eigen::OuterStride<>(ref.outerStride())
+            );
+            
+            const int M = block.cols();
+            const int K = block.rows();
+            ctx_id = ctx_in;
+            
+            viennacl::context ctx(viennacl::ocl::get_context(ctx_id));
+            
+            shptr.reset(new viennacl::matrix<T>(K, M, ctx=ctx));
             
             viennacl::copy(block, *shptr.get());
         };
