@@ -1,6 +1,8 @@
 library(gpuR)
 context("CPU vclVector shared memory")
 
+current_context <- set_device_context("cpu")
+
 # set seed
 set.seed(123)
 
@@ -34,3 +36,4 @@ test_that("Share memory between vclMatrix & vclVector", {
                  info="double elements not equivalent")
 })
 
+setContext(current_context)
