@@ -1,6 +1,8 @@
 library(gpuR)
 context("CPU vclMatrix algebra")
 
+current_context <- set_device_context("cpu")
+
 # set seed
 set.seed(123)
 
@@ -13,25 +15,37 @@ A <- matrix(rnorm(ORDER^2), nrow=ORDER, ncol=ORDER)
 B <- matrix(rnorm(ORDER^2), nrow=ORDER, ncol=ORDER)
 E <- matrix(rnorm(15), nrow=5)
 
-# Single Precision Tests
+# Single Precision tests
 
-test_that("CPU vclMatrix Single Precision Matrix Multiplication", {
+test_that("CPU vclMatrix Single Precision Matrix multiplication", {
     
     has_cpu_skip()
     
     C <- A %*% B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
-    fvclE <- vclMatrix(E, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type = "float")
     
-    fvclC <- fvclA %*% fvclB
+    fgpuC <- fgpuA %*% fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_error(fvclA %*% fvclE, 
+    expect_error(fgpuA %*% fgpuE,
                  info = "error not thrown for non-conformant matrices")
+    
+    fgpuC <- A %*% fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA %*% B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("CPU vclMatrix Single Precision Matrix Subtraction", {
@@ -40,14 +54,55 @@ test_that("CPU vclMatrix Single Precision Matrix Subtraction", {
     
     C <- A - B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type="float")
     
-    fvclC <- fvclA - fvclB
+    fgpuC <- fgpuA - fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
+    expect_error(fgpuA - fgpuE)
+    
+    fgpuC <- fgpuA - B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- A - fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+})
+
+test_that("CPU vclMatrix Single Precision Matrix/Vector Subtraction", {
+    
+    has_cpu_skip()
+    
+    C <- A - c(B)
+    C2 <- c(A) - B
+    
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclVector(c(B), type="float")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuA <- vclVector(c(A), type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C2, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
 })
 
 test_that("CPU vclMatrix Single Precision Scalar Matrix Subtraction", {
@@ -57,16 +112,16 @@ test_that("CPU vclMatrix Single Precision Scalar Matrix Subtraction", {
     C <- A - 1
     C2 <- 1 - A
     
-    fvclA <- vclMatrix(A, type="float")
+    fgpuA <- vclMatrix(A, type="float")
     
-    fvclC <- fvclA - 1    
-    fvclC2 <- 1 - fvclA
+    fgpuC <- fgpuA - 1    
+    fgpuC2 <- 1 - fgpuA
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_is(fvclC2, "fvclMatrix")
-    expect_equal(fvclC2[,], C2, tolerance=1e-07, 
+    expect_is(fgpuC2, "fvclMatrix")
+    expect_equal(fgpuC2[,], C2, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
@@ -76,12 +131,12 @@ test_that("CPU vclMatrix Single Precision Unary Scalar Matrix Subtraction", {
     
     C <- -A
     
-    fvclA <- vclMatrix(A, type="float")
+    fgpuA <- vclMatrix(A, type="float")
     
-    fvclC <- -fvclA
+    fgpuC <- -fgpuA
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
@@ -91,14 +146,54 @@ test_that("CPU vclMatrix Single Precision Matrix Addition", {
     
     C <- A + B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type="float")
     
-    fvclC <- fvclA + fvclB
+    fgpuC <- fgpuA + fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
+    expect_error(fgpuA + fgpuE)
+    
+    fgpuC <- A + fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA + B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+})
+
+test_that("CPU vclMatrix Single Precision Matrix/Vector Addition", {
+    
+    has_cpu_skip()
+    
+    C <- A + c(B)
+    
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclVector(c(B), type="float")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuA <- vclVector(c(A), type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
 })
 
 test_that("CPU vclMatrix Single Precision Scalar Matrix Addition", {
@@ -108,16 +203,16 @@ test_that("CPU vclMatrix Single Precision Scalar Matrix Addition", {
     C <- A + 1
     C2 <- 1 + A
     
-    fvclA <- vclMatrix(A, type="float")
+    fgpuA <- vclMatrix(A, type="float")
     
-    fvclC <- fvclA + 1
-    fvclC2 <- 1 + fvclA
+    fgpuC <- fgpuA + 1
+    fgpuC2 <- 1 + fgpuA
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_is(fvclC2, "fvclMatrix")
-    expect_equal(fvclC2[,], C2, tolerance=1e-07, 
+    expect_is(fgpuC2, "fvclMatrix")
+    expect_equal(fgpuC2[,], C2, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
@@ -127,16 +222,28 @@ test_that("CPU vclMatrix Single Precision Matrix Element-Wise Multiplication", {
     
     C <- A * B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
-    fvclE <- vclMatrix(E, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type="float")
     
-    fvclC <- fvclA * fvclB
+    fgpuC <- fgpuA * fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    expect_error(fgpuA * fgpuE)
+    
+    fgpuC <- A * fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA * B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_error(fvclA * fvclE)
 })
 
 test_that("CPU vclMatrix Single Precision Scalar Matrix Multiplication", {
@@ -146,16 +253,16 @@ test_that("CPU vclMatrix Single Precision Scalar Matrix Multiplication", {
     C <- A * 2
     C2 <- 2 * A
     
-    dvclA <- vclMatrix(A, type="float")
+    dgpuA <- vclMatrix(A, type="float")
     
-    dvclC <- dvclA * 2
-    dvclC2 <- 2 * dvclA
+    dgpuC <- dgpuA * 2
+    dgpuC2 <- 2 * dgpuA
     
-    expect_is(dvclC, "fvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=1e-07, 
+    expect_is(dgpuC, "fvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_is(dvclC2, "fvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=1e-07, 
+    expect_is(dgpuC2, "fvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
@@ -165,16 +272,28 @@ test_that("CPU vclMatrix Single Precision Matrix Element-Wise Division", {
     
     C <- A / B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
-    fvclE <- vclMatrix(E, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type="float")
     
-    fvclC <- fvclA / fvclB
+    fgpuC <- fgpuA / fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    expect_error(fgpuA / fgpuE)
+    
+    fgpuC <- A / fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA / B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_error(fvclA * fvclE)
 })
 
 test_that("CPU vclMatrix Single Precision Scalar Matrix Division", {
@@ -184,35 +303,48 @@ test_that("CPU vclMatrix Single Precision Scalar Matrix Division", {
     C <- A/2
     C2 <- 2/A
     
-    dvclA <- vclMatrix(A, type="float")
+    dgpuA <- vclMatrix(A, type="float")
     
-    dvclC <- dvclA/2
-    dvclC2 <- 2/dvclA
+    dgpuC <- dgpuA/2
+    dgpuC2 <- 2/dgpuA
     
-    expect_is(dvclC, "fvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=1e-07, 
+    expect_is(dgpuC, "fvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_is(dvclC2, "fvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=1e-07, 
+    expect_is(dgpuC2, "fvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Single Precision Matrix Element-Wise Power", {
     
     has_cpu_skip()
+    pocl_check()
     
     C <- A ^ B
     
-    fvclA <- vclMatrix(A, type="float")
-    fvclB <- vclMatrix(B, type="float")
-    fvclE <- vclMatrix(E, type="float")
+    fgpuA <- vclMatrix(A, type="float")
+    fgpuB <- vclMatrix(B, type="float")
+    fgpuE <- vclMatrix(E, type="float")
     
-    fvclC <- fvclA ^ fvclB
+    fgpuC <- fgpuA ^ fgpuB
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    expect_error(fgpuA ^ fgpuE)
+    
+    fgpuC <- A ^ fgpuB
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA ^ B
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_error(fvclA ^ fvclE)
 })
 
 test_that("CPU vclMatrix Single Precision Scalar Matrix Power", {
@@ -220,13 +352,17 @@ test_that("CPU vclMatrix Single Precision Scalar Matrix Power", {
     has_cpu_skip()
     
     C <- A^2
+    C2 <- 2^A
     
-    dvclA <- vclMatrix(A, type="float")
+    dgpuA <- vclMatrix(A, type="float")
     
-    dvclC <- dvclA^2
+    dgpuC <- dgpuA^2
+    dgpuC2 <- 2^dgpuA
     
-    expect_is(dvclC, "fvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=1e-07, 
+    expect_is(dgpuC, "fvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent") 
+    expect_equal(dgpuC2[,], C2, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
 })
 
@@ -241,45 +377,69 @@ test_that("CPU vclMatrix Single Precision crossprod", {
     C <- crossprod(X,Y)
     Cs <- crossprod(X)
     
-    fvclX <- vclMatrix(X, type="float")
-    fvclY <- vclMatrix(Y, type="float")
-    fvclZ <- vclMatrix(Z, type="float")
+    fgpuX <- vclMatrix(X, type="float")
+    fgpuY <- vclMatrix(Y, type="float")
+    fgpuZ <- vclMatrix(Z, type="float")
     
-    fvclC <- crossprod(fvclX, fvclY)
-    fvclCs <- crossprod(fvclX)
+    fgpuC <- crossprod(fgpuX, fgpuY)
+    fgpuCs <- crossprod(fgpuX)
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_equal(fvclCs[,], Cs, tolerance=1e-07, 
+    expect_equal(fgpuCs[,], Cs, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_error(crossprod(fvclX, fvclZ))
+    expect_error(crossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- crossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- crossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("CPU vclMatrix Single Precision tcrossprod", {
     
     has_cpu_skip()
     
-    X <- matrix(rnorm(10), nrow=5)
-    Y <- matrix(rnorm(10), nrow=5)
-    Z <- matrix(rnorm(15), nrow=5)
+    X <- matrix(rnorm(10), nrow=2)
+    Y <- matrix(rnorm(10), nrow=2)
+    Z <- matrix(rnorm(12), nrow=2)
     
     C <- tcrossprod(X,Y)
     Cs <- tcrossprod(X)
     
-    fvclX <- vclMatrix(X, type="float")
-    fvclY <- vclMatrix(Y, type="float")
-    fvclZ <- vclMatrix(Z, type="float")
+    fgpuX <- vclMatrix(X, type="float")
+    fgpuY <- vclMatrix(Y, type="float")
+    fgpuZ <- vclMatrix(Z, type="float")
     
-    fvclC <- tcrossprod(fvclX, fvclY)
-    fvclCs <- tcrossprod(fvclX)
+    fgpuC <- tcrossprod(fgpuX, fgpuY)
+    fgpuCs <- tcrossprod(fgpuX)
     
-    expect_is(fvclC, "fvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=1e-07, 
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
-    expect_equal(fvclCs[,], Cs, tolerance=1e-07, 
+    expect_equal(fgpuCs[,], Cs, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
-    expect_error(tcrossprod(fvclX, fvclZ))
+    expect_error(tcrossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- tcrossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- tcrossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "fvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("CPU vclMatrix Single Precision transpose", {
@@ -297,6 +457,20 @@ test_that("CPU vclMatrix Single Precision transpose", {
 })
 
 
+test_that("vclMatrix Single Precision determinant", {
+    
+    has_cpu_skip()
+    
+    d <- det(A)
+    
+    fgpuA <- vclMatrix(A, type="double")
+    fgpud <- det(fgpuA)
+    
+    expect_is(fgpud, "numeric")
+    expect_equal(fgpud, d, tolerance=1e-07, 
+                 info="float determinants not equivalent") 
+})
+
 # Integer tests
 
 test_that("CPU vclMatrix Integer Matrix multiplication", {
@@ -310,6 +484,16 @@ test_that("CPU vclMatrix Integer Matrix multiplication", {
     
     igpuC <- igpuA %*% igpuB
     
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
+    
+    igpuC <- Aint %*% igpuB
+    
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
+    
+    igpuC <- igpuA %*% Bint
+    
     expect_equivalent(igpuC[,], Cint,
                       info="integer matrix elements not equivalent")
 })
@@ -328,6 +512,20 @@ test_that("CPU vclMatrix Integer Matrix Subtraction", {
     expect_is(igpuC, "ivclMatrix")
     expect_equal(igpuC[,], Cint,
                  info="integer matrix elements not equivalent")
+    
+    
+    igpuC <- igpuA - Bint
+    
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    
+    igpuC <- Aint - igpuB
+    
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
 })
 
 test_that("CPU vclMatrix Integer Precision Scalar Matrix Subtraction", {
@@ -376,6 +574,18 @@ test_that("CPU vclMatrix Integer Matrix Addition", {
     
     igpuC <- igpuA + igpuB
     
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    igpuC <- Aint + igpuB
+    
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    igpuC <- igpuA + Bint
+    
     expect_is(igpuC, "ivclMatrix")
     expect_equal(igpuC[,], Cint,
                  info="integer matrix elements not equivalent")
@@ -414,7 +624,19 @@ test_that("CPU vclMatrix Integer Precision Matrix Element-Wise Multiplication",
     
     expect_is(fgpuC, "ivclMatrix")
     expect_equal(fgpuC[,], C, 
-                 info="integer matrix elements not equivalent")  
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- Aint * fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- fgpuA * Bint
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Integer Precision Scalar Matrix Multiplication", {
@@ -451,7 +673,19 @@ test_that("CPU vclMatrix Integer Precision Matrix Element-Wise Division", {
     
     expect_is(fgpuC, "ivclMatrix")
     expect_equal(fgpuC[,], C,
-                 info="integer matrix elements not equivalent")  
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- Aint / fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- fgpuA / Bint
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Integer Precision Scalar Matrix Division", {
@@ -480,6 +714,7 @@ test_that("CPU vclMatrix Integer Precision Scalar Matrix Division", {
 test_that("CPU vclMatrix Integer Precision Matrix Element-Wise Power", {
     
     has_cpu_skip()
+    pocl_check()
     
     Apow <- matrix(seq.int(9), ncol=3, nrow=3)
     Bpow <- matrix(2, ncol = 3, nrow = 3)
@@ -490,6 +725,18 @@ test_that("CPU vclMatrix Integer Precision Matrix Element-Wise Power", {
     
     fgpuC <- fgpuA ^ fgpuB
     
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    
+    fgpuC <- Apow ^ fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    
+    fgpuC <- fgpuA ^ Bpow
+    
     expect_is(fgpuC, "ivclMatrix")
     expect_equal(fgpuC[,], C,
                  info="integer matrix elements not equivalent")
@@ -517,236 +764,364 @@ test_that("CPU vclMatrix Integer Precision Scalar Matrix Power", {
                  info="integer matrix elements not equivalent")
 })
 
-# Double Precision Tests
+# Double Precision tests
 
 test_that("CPU vclMatrix Double Precision Matrix multiplication", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A %*% B
     
-    dvclA <- vclMatrix(A, type="double")
-    dvclB <- vclMatrix(B, type="double")
+    dgpuA <- vclMatrix(A, type="double")
+    dgpuB <- vclMatrix(B, type="double")
     
-    dvclC <- dvclA %*% dvclB
+    dgpuC <- dgpuA %*% dgpuB
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA %*% B
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- A %*% dgpuB
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
 })
 
 test_that("CPU vclMatrix Double Precision Matrix Subtraction", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A - B
     
-    dvclA <- vclMatrix(A, type="double")
-    dvclB <- vclMatrix(B, type="double")
+    dgpuA <- vclMatrix(A, type="double")
+    dgpuB <- vclMatrix(B, type="double")
+    dgpuE <- vclMatrix(E, type="double")
     
-    dvclC <- dvclA - dvclB
+    dgpuC <- dgpuA - dgpuB
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
+    expect_error(dgpuA - dgpuE)
+    
+    dgpuC <- A - dgpuB
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA - B
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+})
+
+test_that("CPU vclMatrix Double Precision Matrix/Vector Subtraction", {
+    
+    has_cpu_skip()
+    
+    C <- A - c(B)
+    C2 <- c(A) - B
+    
+    fgpuA <- vclMatrix(A, type="double")
+    fgpuB <- vclVector(c(B), type="double")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuA <- vclVector(c(A), type="double")
+    fgpuB <- vclMatrix(B, type="double")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C2, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
 })
 
 test_that("CPU vclMatrix Double Precision Matrix Addition", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A + B
     
-    dvclA <- vclMatrix(A, type="double")
-    dvclB <- vclMatrix(B, type="double")
+    dgpuA <- vclMatrix(A, type="double")
+    dgpuB <- vclMatrix(B, type="double")
+    dgpuE <- vclMatrix(E, type="double")
     
-    dvclC <- dvclA + dvclB
+    dgpuC <- dgpuA + dgpuB
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
+    expect_error(dgpuA + dgpuE)
+    
+    dgpuC <- A + dgpuB
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA + B
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+})
+
+test_that("CPU vclMatrix Double Precision Matrix/Vector Addition", {
+    
+    has_cpu_skip()
+
+    C <- A + c(B)
+    
+    fgpuA <- vclMatrix(A, type="double")
+    fgpuB <- vclVector(c(B), type="double")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuA <- vclVector(c(A), type="double")
+    fgpuB <- vclMatrix(B, type="double")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
 })
 
 test_that("CPU vclMatrix Double Precision Scalar Matrix Addition", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A + 1
     C2 <- 1 + A
     
-    dvclA <- vclMatrix(A, type="double")
+    dgpuA <- vclMatrix(A, type="double")
     
-    dvclC <- dvclA + 1
-    dvclC2 <- 1 + dvclA
+    dgpuC <- dgpuA + 1
+    dgpuC2 <- 1 + dgpuA
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_is(dvclC2, "dvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC2, "dvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Scalar Matrix Subtraction", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A - 1
     C2 <- 1 - A
     
-    dvclA <- vclMatrix(A, type="double")
+    dgpuA <- vclMatrix(A, type="double")
     
-    dvclC <- dvclA - 1
-    dvclC2 <- 1 - dvclA
+    dgpuC <- dgpuA - 1
+    dgpuC2 <- 1 - dgpuA
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_is(dvclC2, "dvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC2, "dvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Unary Matrix Subtraction", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- -A
     
-    fvclA <- vclMatrix(A, type="double")
+    fgpuA <- vclMatrix(A, type="double")
     
-    fvclC <- -fvclA
+    fgpuC <- -fgpuA
     
-    expect_is(fvclC, "dvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Matrix Element-Wise Multiplication", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A * B
     
-    dvclA <- vclMatrix(A, type="double")
-    dvclB <- vclMatrix(B, type="double")
-    dvclE <- vclMatrix(E, type="double")
+    dgpuA <- vclMatrix(A, type="double")
+    dgpuB <- vclMatrix(B, type="double")
+    dgpuE <- vclMatrix(E, type="double")
     
-    dvclC <- dvclA * dvclB
+    dgpuC <- dgpuA * dgpuB
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
-    expect_error(dvclA * dvclE)
+    expect_error(dgpuA * dgpuE)
+    
+    dgpuC <- A * dgpuB
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent") 
+    
+    dgpuC <- dgpuA * B
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Scalar Matrix Multiplication", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A * 2
     C2 <- 2 * A
     
-    dvclA <- vclMatrix(A, type="double")
+    dgpuA <- vclMatrix(A, type="double")
     
-    dvclC <- dvclA * 2
-    dvclC2 <- 2 * dvclA
+    dgpuC <- dgpuA * 2
+    dgpuC2 <- 2 * dgpuA
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_is(dvclC2, "dvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC2, "dvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Matrix Element-Wise Division", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A / B
     
-    dvclA <- vclMatrix(A, type="double")
-    dvclB <- vclMatrix(B, type="double")
-    dvclE <- vclMatrix(E, type="double")
+    dgpuA <- vclMatrix(A, type="double")
+    dgpuB <- vclMatrix(B, type="double")
+    dgpuE <- vclMatrix(E, type="double")
     
-    dvclC <- dvclA / dvclB
+    dgpuC <- dgpuA / dgpuB
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    expect_error(dgpuA * dgpuE)
+    
+    dgpuC <- A / dgpuB
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA / B
+    
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
-    expect_error(dvclA * dvclE)
 })
 
 test_that("CPU vclMatrix Double Precision Scalar Matrix Division", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A/2
     C2 <- 2/A
     
-    dvclA <- vclMatrix(A, type="double")
+    dgpuA <- vclMatrix(A, type="double")
     
-    dvclC <- dvclA/2
-    dvclC2 <- 2/dvclA
+    dgpuC <- dgpuA/2
+    dgpuC2 <- 2/dgpuA
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_is(dvclC2, "dvclMatrix")
-    expect_equal(dvclC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC2, "dvclMatrix")
+    expect_equal(dgpuC2[,], C2, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision Matrix Element-Wise Power", {
     
     has_cpu_skip()
-    has_double_skip()
+    pocl_check()
     
     C <- A ^ B
     
-    fvclA <- vclMatrix(A, type="double")
-    fvclB <- vclMatrix(B, type="double")
-    fvclE <- vclMatrix(E, type="double")
+    fgpuA <- vclMatrix(A, type="double")
+    fgpuB <- vclMatrix(B, type="double")
+    fgpuE <- vclMatrix(E, type="double")
     
-    fvclC <- fvclA ^ fvclB
+    fgpuC <- fgpuA ^ fgpuB
     
-    expect_is(fvclC, "dvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    expect_error(fgpuA ^ fgpuE)
+    
+    fgpuC <- A ^ fgpuB
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA ^ B
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
-    expect_error(fvclA ^ fvclE)
 })
 
 test_that("CPU vclMatrix Double Precision Scalar Matrix Power", {
     
     has_cpu_skip()
-    has_double_skip()
+    
     
     C <- A^2
     
-    dvclA <- vclMatrix(A, type="double")
+    dgpuA <- vclMatrix(A, type="double")
     
-    dvclC <- dvclA^2
+    dgpuC <- dgpuA^2
     
-    expect_is(dvclC, "dvclMatrix")
-    expect_equal(dvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(dgpuC, "dvclMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
 })
 
 test_that("CPU vclMatrix Double Precision crossprod", {
     
     has_cpu_skip()
-    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -755,25 +1130,36 @@ test_that("CPU vclMatrix Double Precision crossprod", {
     C <- crossprod(X,Y)
     Cs <- crossprod(X)
     
-    fvclX <- vclMatrix(X, type="double")
-    fvclY <- vclMatrix(Y, type="double")
-    fvclZ <- vclMatrix(Z, type="double")
+    fgpuX <- vclMatrix(X, type="double")
+    fgpuY <- vclMatrix(Y, type="double")
+    fgpuZ <- vclMatrix(Z, type="double")
     
-    fvclC <- crossprod(fvclX, fvclY)
-    fvclCs <- crossprod(fvclX)
+    fgpuC <- crossprod(fgpuX, fgpuY)
+    fgpuCs <- crossprod(fgpuX)
     
-    expect_is(fvclC, "dvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
-    expect_equal(fvclCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_equal(fgpuCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_error(crossprod(fvclX, fvclZ))
+    expect_error(crossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- crossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuC <- crossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 test_that("CPU vclMatrix Double Precision tcrossprod", {
     
     has_cpu_skip()
-    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -782,34 +1168,31 @@ test_that("CPU vclMatrix Double Precision tcrossprod", {
     C <- tcrossprod(X,Y)
     Cs <- tcrossprod(X)
     
-    fvclX <- vclMatrix(X, type="double")
-    fvclY <- vclMatrix(Y, type="double")
-    fvclZ <- vclMatrix(Z, type="double")
+    fgpuX <- vclMatrix(X, type="double")
+    fgpuY <- vclMatrix(Y, type="double")
+    fgpuZ <- vclMatrix(Z, type="double")
     
-    fvclC <- tcrossprod(fvclX, fvclY)
-    fvclCs <- tcrossprod(fvclX)
+    fgpuC <- tcrossprod(fgpuX, fgpuY)
+    fgpuCs <- tcrossprod(fgpuX)
     
-    expect_is(fvclC, "dvclMatrix")
-    expect_equal(fvclC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
-    expect_equal(fvclCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
+    expect_equal(fgpuCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
-    expect_error(tcrossprod(fvclX, fvclZ))
-})
-
-test_that("CPU vclMatrix Double Precision transpose", {
+    expect_error(tcrossprod(fgpuX, fgpuZ))
     
-    has_cpu_skip()
-    has_double_skip()
+    fgpuC <- tcrossprod(fgpuX, Y)
     
-    At <- t(A)
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
     
-    fgpuA <- vclMatrix(A, type="double")
-    fgpuAt <- t(fgpuA)
+    fgpuC <- tcrossprod(X, fgpuY)
     
-    expect_is(fgpuAt, "dvclMatrix")
-    expect_equal(fgpuAt[,], At, tolerance=.Machine$double.eps^0.5, 
-                 info="transposed double matrix elements not equivalent") 
+    expect_is(fgpuC, "dvclMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 
@@ -849,3 +1232,20 @@ test_that("CPU vclMatrix Diagonal access", {
     expect_equal(fgpuA[,], A, tolerance=.Machine$double.eps^0.5, 
                  info="set double matrix diagonal elements not equivalent") 
 })
+
+test_that("vclMatrix Double Precision determinant", {
+    
+    has_cpu_skip()
+    
+    d <- det(A)
+    
+    fgpuA <- vclMatrix(A, type="double")
+    fgpud <- det(fgpuA)
+    
+    expect_is(fgpud, "numeric")
+    expect_equal(fgpud, d, tolerance=.Machine$double.eps^0.5, 
+                 info="double determinants not equivalent") 
+})
+
+setContext(current_context)
+
