@@ -1,6 +1,12 @@
 library(gpuR)
 context("gpuMatrix algebra")
 
+if(detectGPUs() >= 1){
+    current_context <- set_device_context("gpu")    
+}else{
+    current_context <- currentContext()
+}
+
 # set seed
 set.seed(123)
 
@@ -32,6 +38,18 @@ test_that("gpuMatrix Single Precision Matrix multiplication", {
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA %*% fgpuE,
                  info = "error not thrown for non-conformant matrices")
+    
+    fgpuC <- A %*% fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA %*% B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision Matrix Subtraction", {
@@ -43,13 +61,52 @@ test_that("gpuMatrix Single Precision Matrix Subtraction", {
     fgpuA <- gpuMatrix(A, type="float")
     fgpuB <- gpuMatrix(B, type="float")
     fgpuE <- gpuMatrix(E, type="float")
-
+    
     fgpuC <- fgpuA - fgpuB
     
     expect_is(fgpuC, "fgpuMatrix")
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA - fgpuE)
+    
+    fgpuC <- fgpuA - B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- A - fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+})
+
+test_that("gpuMatrix Single Precision Matrix/Vector Subtraction", {
+    
+    has_gpu_skip()
+    
+    C <- A - c(B)
+    C2 <- c(A) - B
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    fgpuB <- gpuVector(c(B), type="float")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuA <- gpuVector(c(A), type="float")
+    fgpuB <- gpuMatrix(B, type="float")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C2, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
 })
 
 test_that("gpuMatrix Single Precision Scalar Matrix Subtraction", {
@@ -103,6 +160,44 @@ test_that("gpuMatrix Single Precision Matrix Addition", {
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA + fgpuE)
+    
+    fgpuC <- A + fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA + B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+})
+
+test_that("gpuMatrix Single Precision Matrix/Vector Addition", {
+    
+    has_gpu_skip()
+    
+    C <- A + c(B)
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    fgpuB <- gpuVector(c(B), type="float")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuA <- gpuVector(c(A), type="float")
+    fgpuB <- gpuMatrix(B, type="float")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
 })
 
 test_that("gpuMatrix Single Precision Scalar Matrix Addition", {
@@ -141,6 +236,18 @@ test_that("gpuMatrix Single Precision Matrix Element-Wise Multiplication", {
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA * fgpuE)
+    
+    fgpuC <- A * fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA * B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision Scalar Matrix Multiplication", {
@@ -179,6 +286,18 @@ test_that("gpuMatrix Single Precision Matrix Element-Wise Division", {
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA / fgpuE)
+    
+    fgpuC <- A / fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA / B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision Scalar Matrix Division", {
@@ -204,6 +323,7 @@ test_that("gpuMatrix Single Precision Scalar Matrix Division", {
 test_that("gpuMatrix Single Precision Matrix Element-Wise Power", {
     
     has_gpu_skip()
+    pocl_check()
     
     C <- A ^ B
     
@@ -217,6 +337,18 @@ test_that("gpuMatrix Single Precision Matrix Element-Wise Power", {
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
     expect_error(fgpuA ^ fgpuE)
+    
+    fgpuC <- A ^ fgpuB
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA ^ B
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision Scalar Matrix Power", {
@@ -262,6 +394,18 @@ test_that("gpuMatrix Single Precision crossprod", {
     expect_equal(fgpuCs[,], Cs, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
     expect_error(crossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- crossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- crossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision tcrossprod", {
@@ -288,6 +432,18 @@ test_that("gpuMatrix Single Precision tcrossprod", {
     expect_equal(fgpuCs[,], Cs, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
     expect_error(tcrossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- tcrossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
+    
+    fgpuC <- tcrossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "fgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=1e-07, 
+                 info="float matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Single Precision transpose", {
@@ -304,34 +460,72 @@ test_that("gpuMatrix Single Precision transpose", {
                  info="transposed float matrix elements not equivalent") 
 })
 
+test_that("gpuMatrix Single Precision determinant", {
+    
+    has_gpu_skip()
+    
+    d <- det(A)
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    fgpud <- det(fgpuA)
+    
+    expect_is(fgpud, "numeric")
+    expect_equal(fgpud, d, tolerance=1e-07, 
+                 info="float determinants not equivalent") 
+})
+
 # Integer tests
 
 test_that("gpuMatrix Integer Matrix multiplication", {
-
+    
     has_gpu_skip()
-
+    
     Cint <- Aint %*% Bint
-
+    
     igpuA <- gpuMatrix(Aint, type="integer")
     igpuB <- gpuMatrix(Bint, type="integer")
-
+    
     igpuC <- igpuA %*% igpuB
-
+    
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
+    
+    igpuC <- Aint %*% igpuB
+    
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
+    
+    igpuC <- igpuA %*% Bint
+    
     expect_equivalent(igpuC[,], Cint,
                       info="integer matrix elements not equivalent")
 })
 
 test_that("gpuMatrix Integer Matrix Subtraction", {
-
+    
     has_gpu_skip()
-
+    
     Cint <- Aint - Bint
-
+    
     igpuA <- gpuMatrix(Aint, type="integer")
     igpuB <- gpuMatrix(Bint, type="integer")
-
+    
     igpuC <- igpuA - igpuB
-
+    
+    expect_is(igpuC, "igpuMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    
+    igpuC <- igpuA - Bint
+    
+    expect_is(igpuC, "igpuMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    
+    igpuC <- Aint - igpuB
+    
     expect_is(igpuC, "igpuMatrix")
     expect_equal(igpuC[,], Cint,
                  info="integer matrix elements not equivalent")
@@ -383,6 +577,18 @@ test_that("gpuMatrix Integer Matrix Addition", {
     
     igpuC <- igpuA + igpuB
     
+    expect_is(igpuC, "igpuMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    igpuC <- Aint + igpuB
+    
+    expect_is(igpuC, "igpuMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+    
+    igpuC <- igpuA + Bint
+    
     expect_is(igpuC, "igpuMatrix")
     expect_equal(igpuC[,], Cint,
                  info="integer matrix elements not equivalent")
@@ -421,7 +627,19 @@ test_that("gpuMatrix Integer Precision Matrix Element-Wise Multiplication", {
     
     expect_is(fgpuC, "igpuMatrix")
     expect_equal(fgpuC[,], C, 
-                 info="integer matrix elements not equivalent")  
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- Aint * fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- fgpuA * Bint
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("gpuMatrix Integer Precision Scalar Matrix Multiplication", {
@@ -458,7 +676,19 @@ test_that("gpuMatrix Integer Precision Matrix Element-Wise Division", {
     
     expect_is(fgpuC, "igpuMatrix")
     expect_equal(fgpuC[,], C,
-                 info="integer matrix elements not equivalent")  
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- Aint / fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    
+    fgpuC <- fgpuA / Bint
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("gpuMatrix Integer Precision Scalar Matrix Division", {
@@ -485,38 +715,51 @@ test_that("gpuMatrix Integer Precision Scalar Matrix Division", {
 })
 
 test_that("gpuMatrix Integer Precision Matrix Element-Wise Power", {
-
+    
     has_gpu_skip()
-
+    pocl_check()
+    
     Apow <- matrix(seq.int(9), ncol=3, nrow=3)
     Bpow <- matrix(2, ncol = 3, nrow = 3)
     C <- Apow ^ Bpow
-
+    
     fgpuA <- gpuMatrix(Apow, type="integer")
     fgpuB <- gpuMatrix(Bpow, type="integer")
-
+    
     fgpuC <- fgpuA ^ fgpuB
-
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    
+    fgpuC <- Apow ^ fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    
+    fgpuC <- fgpuA ^ Bpow
+    
     expect_is(fgpuC, "igpuMatrix")
     expect_equal(fgpuC[,], C,
                  info="integer matrix elements not equivalent")
 })
 
 test_that("gpuMatrix Integer Precision Scalar Matrix Power", {
-
+    
     has_gpu_skip()
-
+    
     C <- Aint^2L
     C2 <- 2L^Aint
-
+    
     C <- apply(C, 2, as.integer)
     C2 <- apply(C2, 2, as.integer)
-
+    
     dgpuA <- gpuMatrix(Aint, type="integer")
-
+    
     dgpuC <- dgpuA^2L
     dgpuC2 <- 2L^dgpuA
-
+    
     expect_is(dgpuC, "igpuMatrix")
     expect_equal(dgpuC[,], C,
                  info="integer matrix elements not equivalent")
@@ -524,67 +767,6 @@ test_that("gpuMatrix Integer Precision Scalar Matrix Power", {
                  info="integer matrix elements not equivalent")
 })
 
-# test_that("gpuMatrix Integer Precision crossprod", {
-#     
-#     has_gpu_skip()
-#     
-#     X <- matrix(seq.int(10), nrow=2)
-#     Y <- matrix(seq.int(10), nrow=2)
-#     
-#     C <- crossprod(X,Y)
-#     Cs <- crossprod(X)
-#     
-#     fgpuX <- gpuMatrix(X, type="integer")
-#     fgpuY <- gpuMatrix(Y, type="integer")
-#     
-#     fgpuC <- crossprod(fgpuX, fgpuY)
-#     fgpuCs <- crossprod(fgpuX)
-#     
-#     expect_is(fgpuC, "igpuMatrix")
-#     expect_equal(fgpuC[,], C,
-#                  info="integer matrix elements not equivalent")  
-#     expect_equal(fgpuCs[,], Cs,
-#                  info="integer matrix elements not equivalent") 
-# })
-# 
-# test_that("gpuMatrix Integer Precision tcrossprod", {
-#     
-#     has_gpu_skip()
-#     
-#     X <- matrix(seq.int(10), nrow=2)
-#     Y <- matrix(seq.int(10), nrow=2)
-#     
-#     C <- tcrossprod(X,Y)
-#     Cs <- tcrossprod(X)
-#     
-#     fgpuX <- gpuMatrix(X, type="integer")
-#     fgpuY <- gpuMatrix(Y, type="integer")
-#     fgpuZ <- gpuMatrix(Z, type="integer")
-#     
-#     fgpuC <- tcrossprod(fgpuX, fgpuY)
-#     fgpuCs <- tcrossprod(fgpuX)
-#     
-#     expect_is(fgpuC, "igpuMatrix")
-#     expect_equal(fgpuC[,], C,
-#                  info="integer matrix elements not equivalent")  
-#     expect_equal(fgpuCs[,], Cs,
-#                  info="integer matrix elements not equivalent") 
-# })
-
-# test_that("gpuMatrix Integer Precision transpose", {
-# 
-#     has_gpu_skip()
-# 
-#     At <- t(Aint)
-# 
-#     fgpuA <- gpuMatrix(Aint, type="integer")
-#     fgpuAt <- t(fgpuA)
-# 
-#     expect_is(fgpuAt, "igpuMatrix")
-#     expect_equal(fgpuAt[,], At,
-#                  info="transposed integer matrix elements not equivalent")
-# })
-
 # Double Precision tests
 
 test_that("gpuMatrix Double Precision Matrix multiplication", {
@@ -599,6 +781,18 @@ test_that("gpuMatrix Double Precision Matrix multiplication", {
     
     dgpuC <- dgpuA %*% dgpuB
     
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA %*% B
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- A %*% dgpuB
+    
     expect_is(dgpuC, "dgpuMatrix")
     expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
@@ -609,6 +803,7 @@ test_that("gpuMatrix Double Precision Matrix Subtraction", {
     has_gpu_skip()
     has_double_skip()
     
+    
     C <- A - B
     
     dgpuA <- gpuMatrix(A, type="double")
@@ -621,6 +816,46 @@ test_that("gpuMatrix Double Precision Matrix Subtraction", {
     expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
     expect_error(dgpuA - dgpuE)
+    
+    dgpuC <- A - dgpuB
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA - B
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+})
+
+test_that("gpuMatrix Double Precision Matrix/Vector Subtraction", {
+    
+    has_gpu_skip()
+    has_double_skip()
+    
+    C <- A - c(B)
+    C2 <- c(A) - B
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    fgpuB <- gpuVector(c(B), type="double")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuA <- gpuVector(c(A), type="double")
+    fgpuB <- gpuMatrix(B, type="double")
+    
+    fgpuC <- fgpuA - fgpuB
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C2, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
 })
 
 test_that("gpuMatrix Double Precision Matrix Addition", {
@@ -640,6 +875,45 @@ test_that("gpuMatrix Double Precision Matrix Addition", {
     expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
     expect_error(dgpuA + dgpuE)
+    
+    dgpuC <- A + dgpuB
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA + B
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+})
+
+test_that("gpuMatrix Double Precision Matrix/Vector Addition", {
+    
+    has_gpu_skip()
+    has_double_skip()
+    
+    C <- A + c(B)
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    fgpuB <- gpuVector(c(B), type="double")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuA <- gpuVector(c(A), type="double")
+    fgpuB <- gpuMatrix(B, type="double")
+    
+    fgpuC <- fgpuA + fgpuB
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix elements not equivalent")  
+    
 })
 
 test_that("gpuMatrix Double Precision Scalar Matrix Addition", {
@@ -717,6 +991,18 @@ test_that("gpuMatrix Double Precision Matrix Element-Wise Multiplication", {
     expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
     expect_error(dgpuA * dgpuE)
+    
+    dgpuC <- A * dgpuB
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent") 
+    
+    dgpuC <- dgpuA * B
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent") 
 })
 
 test_that("gpuMatrix Double Precision Scalar Matrix Multiplication", {
@@ -757,6 +1043,18 @@ test_that("gpuMatrix Double Precision Matrix Element-Wise Division", {
     expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
     expect_error(dgpuA * dgpuE)
+    
+    dgpuC <- A / dgpuB
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    dgpuC <- dgpuA / B
+    
+    expect_is(dgpuC, "dgpuMatrix")
+    expect_equal(dgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Double Precision Scalar Matrix Division", {
@@ -784,6 +1082,7 @@ test_that("gpuMatrix Double Precision Matrix Element-Wise Power", {
     
     has_gpu_skip()
     has_double_skip()
+    pocl_check()
     
     C <- A ^ B
     
@@ -797,6 +1096,18 @@ test_that("gpuMatrix Double Precision Matrix Element-Wise Power", {
     expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent")  
     expect_error(fgpuA ^ fgpuE)
+    
+    fgpuC <- A ^ fgpuB
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuC <- fgpuA ^ B
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Double Precision Scalar Matrix Power", {
@@ -840,6 +1151,18 @@ test_that("gpuMatrix Double Precision crossprod", {
     expect_equal(fgpuCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
     expect_error(crossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- crossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuC <- crossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Double Precision tcrossprod", {
@@ -867,6 +1190,18 @@ test_that("gpuMatrix Double Precision tcrossprod", {
     expect_equal(fgpuCs[,], Cs, tolerance=.Machine$double.eps ^ 0.5, 
                  info="double matrix elements not equivalent") 
     expect_error(tcrossprod(fgpuX, fgpuZ))
+    
+    fgpuC <- tcrossprod(fgpuX, Y)
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
+    
+    fgpuC <- tcrossprod(X, fgpuY)
+    
+    expect_is(fgpuC, "dgpuMatrix")
+    expect_equal(fgpuC[,], C, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="double matrix elements not equivalent")  
 })
 
 test_that("gpuMatrix Double Precision transpose", {
@@ -923,3 +1258,20 @@ test_that("gpuMatrix Diagonal access", {
     expect_equal(fgpuA[,], A, tolerance=.Machine$double.eps^0.5, 
                  info="set double matrix diagonal elements not equivalent") 
 })
+
+test_that("gpuMatrix Double Precision determinant", {
+    
+    has_gpu_skip()
+    has_double_skip()
+    
+    d <- det(A)
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    fgpud <- det(fgpuA)
+    
+    expect_is(fgpud, "numeric")
+    expect_equal(fgpud, d, tolerance=.Machine$double.eps^0.5, 
+                 info="double determinants not equivalent") 
+})
+
+setContext(current_context)
