@@ -1,6 +1,12 @@
 library(gpuR)
 context("Switching GPUs vclMatrix Correlations")
 
+if(detectGPUs() >= 1){
+    current_context <- set_device_context("gpu")    
+}else{
+    current_context <- currentContext()
+}
+
 # set seed
 set.seed(123)
 
@@ -56,3 +62,5 @@ test_that("Switching GPUs vclMatrix Double Precision Pearson Covariance",
     expect_equal(currentContext(), 1L, 
                  info = "context index has been change unintentionally")
 })
+
+setContext(current_context)
