@@ -1,6 +1,12 @@
 library(gpuR)
 context("vclVector math operations")
 
+if(detectGPUs() >= 1){
+    current_context <- set_device_context("gpu")    
+}else{
+    current_context <- currentContext()
+}
+
 # set seed
 set.seed(123)
 
@@ -113,6 +119,7 @@ test_that("vclVector Double Precision Matrix Element-Wise Trignometry", {
 test_that("vclVector Single Precision Matrix Element-Wise Logs", {
     
     has_gpu_skip()
+    pocl_check()
     
     R_log <- suppressWarnings(log(A))
     R_log10 <- suppressWarnings(log10(A))
@@ -139,6 +146,7 @@ test_that("vclVector Double Precision Matrix Element-Wise Logs", {
     
     has_gpu_skip()
     has_double_skip()
+    pocl_check()
     
     R_log <- suppressWarnings(log(A))
     R_log10 <- suppressWarnings(log10(A))
@@ -263,3 +271,133 @@ test_that("vclVector Double Precision Maximum/Minimum", {
                  info="min double vector element not equivalent")  
 })
 
+test_that("vclVector Single Precision pmax/pmin", {
+    
+    has_gpu_skip()
+    
+    R_max <- pmax(A, 0)
+    R_min <- pmin(A, 0)
+    
+    fgpuA <- vclVector(A, type="float")
+    
+    fgpu_max <- pmax(fgpuA, 0)
+    fgpu_min <- pmin(fgpuA, 0)
+    
+    expect_is(fgpu_max, "fvclVector")
+    expect_equal(fgpu_max[], R_max, tolerance=1e-07, 
+                 info="max float vector element not equivalent")  
+    expect_equal(fgpu_min[], R_min, tolerance=1e-07, 
+                 info="min float vector element not equivalent")  
+    
+    # multiple operations
+    R_max <- pmax(A, 0, .5)
+    R_min <- pmin(A, 0, -.2)
+    
+    fgpu_max <- pmax(fgpuA, 0, 0.5)
+    fgpu_min <- pmin(fgpuA, 0, -.2)
+    
+    expect_is(fgpu_max, "fvclVector")
+    expect_equal(fgpu_max[], R_max, tolerance=1e-07, 
+                 info="max float vector element not equivalent")  
+    expect_equal(fgpu_min[], R_min, tolerance=1e-07, 
+                 info="min float vector element not equivalent") 
+})
+
+test_that("vclVector Double Precision pmax/pmin", {
+    
+    has_gpu_skip()
+    has_double_skip()
+    
+    R_max <- pmax(A, 0)
+    R_min <- pmin(A, 0)
+    
+    fgpuA <- vclVector(A, type="double")
+    
+    fgpu_max <- pmax(fgpuA, 0)
+    fgpu_min <- pmin(fgpuA, 0)
+    
+    expect_is(fgpu_max, "dvclVector")
+    expect_equal(fgpu_max[], R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double vector element not equivalent") 
+    expect_equal(fgpu_min[], R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double vector element not equivalent")  
+    
+    
+    # multiple operations
+    R_max <- pmax(A, 0, .5)
+    R_min <- pmin(A, 0, -.2)
+    
+    fgpu_max <- pmax(fgpuA, 0, 0.5)
+    fgpu_min <- pmin(fgpuA, 0, -.2)
+    
+    expect_is(fgpu_max, "dvclVector")
+    expect_equal(fgpu_max[], R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double vector element not equivalent") 
+    expect_equal(fgpu_min[], R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double vector element not equivalent")  
+})
+
+test_that("vclVector Single Precision sqrt", {
+    
+    has_gpu_skip()
+    
+    R_sqrt <- sqrt(abs(A))
+    
+    fvclA <- vclVector(abs(A), type="float")
+    
+    fvcl_sqrt <- sqrt(fvclA)
+    
+    expect_is(fvcl_sqrt, "fvclVector")
+    expect_equal(fvcl_sqrt[,], R_sqrt, tolerance=1e-07, 
+                 info="sqrt float vector elements not equivalent")  
+})
+
+test_that("vclVector Double Precision sqrt", {
+    
+    has_gpu_skip()
+    has_double_skip()
+    
+    R_sqrt <- sqrt(abs(A))
+    
+    fvclA <- vclVector(abs(A), type="double")
+    
+    fvcl_sqrt <- sqrt(fvclA)
+    
+    expect_is(fvcl_sqrt, "dvclVector")
+    expect_equal(fvcl_sqrt[,], R_sqrt, tolerance=.Machine$double.eps^0.5, 
+                 info="sqrt double vector elements not equivalent")  
+})
+
+
+test_that("vclVector Single Precision Matrix sign", {
+    has_gpu_skip()
+    
+    R_sign <- sign(A)
+    
+    fgpuA <- vclVector(A, type="float")
+    
+    fgpu_sign <- sign(fgpuA)
+    
+    expect_is(fgpu_sign, "fvclVector")
+    expect_equal(fgpu_sign[,], R_sign, tolerance=1e-07, 
+                 info="sign float matrix elements not equivalent",
+                 check.attributes=FALSE)  
+})
+
+test_that("vclVector Double Precision Matrix sign", {
+    has_gpu_skip()
+    has_double_skip()
+    
+    R_sign <- sign(A)
+    
+    fgpuA <- vclVector(A, type="double")
+    
+    fgpu_sign <- sign(fgpuA)
+    
+    expect_is(fgpu_sign, "dvclVector")
+    expect_equal(fgpu_sign[,], R_sign, tolerance=.Machine$double.base^0.5, 
+                 info="sign double matrix elements not equivalent",
+                 check.attributes=FALSE)  
+})
+
+setContext(current_context)
